// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMAT_OBJECTS__ARRAY_OUTPUT__HPP
#define BOOST__IO__FORMAT_OBJECTS__ARRAY_OUTPUT__HPP
#  include <boost/io/format_objects/detail/list_output.hpp>

   namespace boost { namespace io
   {
      template< typename FormatType, typename Outputter = boost::io::basic_output >
      class array_output: public detail::list_output< FormatType, array_output
                                                                  <
                                                                     FormatType,
                                                                     Outputter
                                                                  >, Outputter >
      {
         private:
            long                       length;
            long                       offset;
         public:
            template< typename T, class OutputStream >
            inline OutputStream & operator()
                                  (
                                     OutputStream & os,
                                     const T *      array
                                  ) const
            {
               return( detail::list_output< FormatType, array_output, Outputter >::operator()
               (
                  os, array + offset, array + offset + length
               ));
            }
         public:
            inline array_output & operator()( long len );
            inline array_output & operator()( long off, long len );
         public:
            inline           array_output( long len = 0 ) throw();
            inline           array_output( long off, long len ) throw();
            inline           array_output( const Outputter &, long len = 0 ) throw();
            inline           array_output( const Outputter &, long off, long len ) throw();
            inline           array_output( const array_output & ) throw();
            template< typename RT2 >
            inline           array_output
                             (
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               detail::list_output< FormatType, array_output, Outputter >( fmt )
            {
            }
      };

      // array_output generators

      inline array_output< char >                arrayfmt( long l = 0 );
      inline array_output< char >                arrayfmt( long o, long l );

      template< typename FormatType >
      inline array_output< FormatType >          arrayfmtex( long l = 0 );
      template< typename FormatType >
      inline array_output< FormatType >          arrayfmtex( long o, long l );

      template< class Outputter >
      inline array_output< typename Outputter::listfmt_type, Outputter >
                                                 arrayfmtout
                                                 (
                                                    const Outputter & out,
                                                    long l = 0
                                                 );
      template< class Outputter >
      inline array_output< typename Outputter::listfmt_type, Outputter >
                                                 arrayfmtout
                                                 (
                                                    const Outputter & out,
                                                    long o,
                                                    long l
                                                 );
   }}

#  include "detail/array_output.ipp"
#endif
