// (C) Copyright 2003: Reece H. Dunn 

#ifndef BOOST__IO__FORMATLIST__HPP
#define BOOST__IO__FORMATLIST__HPP
#  include <boost/io/format_objects.hpp>
#  include <iostream>
#  include <iterator>

   namespace boost { namespace io
   {
      template
      <
         typename ForwardIterator, typename FormatType = char,
         typename Outputter = boost::io::basic_output
      >
      class formatlist_t: public detail::list_output< FormatType, formatlist_t
                                                                  <
                                                                     ForwardIterator,
                                                                     FormatType,
                                                                     Outputter
                                                                  >, Outputter >
      {
         private:
            ForwardIterator            first;
            ForwardIterator            last;
         public:
            inline ForwardIterator               begin() const throw();
            inline ForwardIterator               end()   const throw();
         public:
            typedef ForwardIterator                        iterator;
         public: // constructors
            inline           formatlist_t( ForwardIterator f, ForwardIterator l ) throw();
            inline           formatlist_t( ForwardIterator f, ForwardIterator l, Outputter ) throw();
            template< typename RT2 >
            inline           formatlist_t
                             (
                                ForwardIterator f,
                                ForwardIterator l,
                                const formatter_t< FormatType, RT2 > & fmt
                             ) throw():
               detail::list_output< FormatType, formatlist_t, Outputter >( fmt ),
               first( f ),
               last(  l )
            {
            }
            template< typename RT2 >
            inline           formatlist_t
                             (
                                ForwardIterator f,
                                ForwardIterator l,
                                const formatter_t< FormatType, RT2 > & fmt,
                                Outputter out
                             ) throw():
               detail::list_output< FormatType, formatlist_t, Outputter >( fmt, out ),
               first( f ),
               last(  l )
            {
            }
      };

      template< typename CharT, class TraitsT, typename ForwardIterator, typename FormatType, typename Outputter >
      inline std::basic_ostream< CharT, TraitsT > & operator<<
      (
         std::basic_ostream< CharT, TraitsT > &,
         const formatlist_t< ForwardIterator, FormatType, Outputter > &
      );

      // basic format manipulator:
      // NOTE: If you want a wchar_t formatter, you must use formatlistex or a formatter object.

      template< typename ForwardIterator >
      inline formatlist_t< ForwardIterator >     formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< class Container >
      inline formatlist_t< typename Container::iterator >
                                                 formatlist
                                                 (
                                                    Container &
                                                 );

      // advanced format manipulator:

      template< typename ForwardIterator, typename FormatType, class RefType >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    const formatter_t< FormatType, RefType > & 
                                                 );

      template< class Container, typename FormatType, class RefType >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlist
                                                 (
                                                    Container &,
                                                    const formatter_t< FormatType, RefType > &
                                                 );

      // extended format manipulator:

      template< typename FormatType, typename ForwardIterator >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlistex
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< typename FormatType, class Container >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlistex
                                                 (
                                                    Container & 
                                                 );

      // format manipulator -- output functor versions:

      template< typename ForwardIterator, class Outputter >
      inline formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    Outputter
                                                 );
      
      template< class Container, class Outputter >
      inline formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    Container &,
                                                    Outputter
                                                 );

      template< typename ForwardIterator, class Outputter, class RefType >
      inline formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    Outputter,
                                                    const formatter_t< typename Outputter::listfmt_type, RefType > &
                                                 );

      template< class Container, class Outputter, class RefType >
      inline formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    Container &,
                                                    Outputter,
                                                    const formatter_t< typename Outputter::listfmt_type, RefType > &
                                                 );
   }}

#  include "detail/formatlist.ipp"
#endif
