// (C) Copyright 2003: Reece H. Dunn

// Test file for boost/io/formatlist.hpp

#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

int main()
{
   // std::vector

   std::vector< int >                  v( 10 );
   std::generate( v.begin(), v.end(), std::rand );

   // std::list

   std::list< char >                   l;
   l.push_back( 'A' );
   l.push_back( 'B' );
   l.push_back( 'C' );

   // std::string and arrays

   const char                        * str  = "Jeanette Biedermann";
   std::string                         s    = "Warum bist du nicht da?";

   int                                 i[ 4 ] = { 3, 6, 9, 12 };

   // formatter configurations

   boost::io::formatter< char >        fmt;
   fmt.format( '(', ')' );
   fmt.space(  true, true );

   boost::io::formatter< char * >      fmt2;
   fmt2.format( "--" );
   fmt2.space(  true, true );

   // examples

   // Mechanism #1: pass a formatter object to the formatlist object:

   std::cout << "mechanism 1:" << '\n';

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( i, i + 4 ).format( fmt )
             << '\n';

   // [results]: int[ 4 ] = ( 3, 6, 9, 12 )

   std::cout << "char * = "
             << boost::io::formatlist( str, str + std::strlen( str )).format( fmt )
             << '\n';

   // [results]: char * = ( J, e, a, n, e, t, t, e,  , B, i, e, d, e, r, m, a, n, n )

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( l ).format( fmt ) // format only
             << '\n';

   // [results]: int[ 4 ] = ( A, B, C )

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( l ).space( fmt ) // spacing only
             << '\n';

   // [results]: int[ 4 ] = [ A , B , C ]

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( l ).inherit( fmt ) // format *and* spacing
             << '\n';

   // [results]: int[ 4 ] = ( A , B , C )

   // Mechanism #2: pass a formatter object to the formatlist manipulator:

   std::cout << '\n' << "mechanism 1 and 2 side-by-side:" << '\n';

   // NOTE: These two are equivalent, but the second will be more efficient
   // and has less typing

   std::cout << "vector< int > = "
             << boost::io::formatlist( v ).inherit( fmt )
             << '\n';

   // [results]: vector< int > = ( 41 , 18467 , 6334 , 26500 , 19169 , 15724 , 11478 , 29358 , 26962 , 24464 )

   std::cout << "vector< int > = "
             << boost::io::formatlist( v, fmt )
             << '\n';

   // [results]: vector< int > = ( 41 , 18467 , 6334 , 26500 , 19169 , 15724 , 11478 , 29358 , 26962 , 24464 )

   // These two are also equivalent:

   std::cout << "int[ 4 ] = "
             << boost::io::formatlistex< char * >( i, i + 4 ).inherit( fmt2 )
             << '\n';

   // [results]: int[ 4 ] = [ 3 -- 6 -- 9 -- 12 ]

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( i, i + 4, fmt2 ) // [note]: inherit format type (char *)
             << '\n';                                   // from fmt2

   // [results]: int[ 4 ] = [ 3 -- 6 -- 9 -- 12 ]

   return( 0 );
}
