// (C) Copyright 2003: Reece H. Dunn 

// Test file for boost/io/formatlist.hpp

#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

int main()
{
   // 3D array

   char                                a3D[ 3 ][ 3 ][ 3 ] =
   {
      {
         { 'A', '0', '0' },
         { '0', 'A', '0' },
         { '0', '0', 'A' }
      }, {
         { 'B', '0', '0' },
         { '0', 'B', '0' },
         { '0', '0', 'B' }
      }, {
         { 'C', '0', '0' },
         { '0', 'C', '0' },
         { '0', '0', 'C' }
      },
   };

   // formatter configurations

   boost::io::array_output< char >     arrayfmt;
   arrayfmt.format( '(', ')' );

   boost::io::array_output< char, boost::io::array_output< char > >
                                       array3Dfmt( arrayfmt( 3 ));
   array3Dfmt.format( '{', '}', '|' ).space( true, true, true );

   // example using formatter objects

   std::cout << "char[ 3 ][ 3 ][ 3 ] = "
             << boost::io::formatlistfn( a3D, a3D + 3, array3Dfmt( 3 ))
                .format( '\n', '\0', '\n' ).space( false, false, false )
             << '\n' << '\n';

   /* [results]:

      char[ 3 ][ 3 ][ 3 ] =
      { ( A, 0, 0 ) | ( 0, A, 0 ) | ( 0, 0, A ) }
      { ( B, 0, 0 ) | ( 0, B, 0 ) | ( 0, 0, B ) }
      { ( C, 0, 0 ) | ( 0, C, 0 ) | ( 0, 0, C ) }
   */

   // partially inlined form

   std::cout << "char[ 3 ][ 3 ][ 3 ] = "
             << boost::io::formatlistfn
                (
                   a3D, a3D + 3,
                   boost::io::arrayfmtout( arrayfmt( 3 ), 3 ) // inlined form
                   .format( '<', '>', ':' ).space( true, true, true )
                )
                .format( '\n', '\0', '\n' ).space( false, false, false )
             << '\n' << '\n';

   /* [results]:

      char[ 3 ][ 3 ][ 3 ] =
      < ( A, 0, 0 ) : ( 0, A, 0 ) : ( 0, 0, A ) >
      < ( B, 0, 0 ) : ( 0, B, 0 ) : ( 0, 0, B ) >
      < ( C, 0, 0 ) : ( 0, C, 0 ) : ( 0, 0, C ) >
   */

   // fully inlined 3D example
   // [note]: using the generator functions is easier for the more complex constructs
   // as they will deduce the object types for you!

   std::cout << "char[ 3 ][ 3 ][ 3 ] = "
             << boost::io::formatlistfn
                (
                   a3D, a3D + 3, // outermost index
                   boost::io::arrayfmtout
                   (
                      boost::io::arrayfmt( 3 ), // innermost index
                      3 // middle index
                   )
                   .format( '<', '>', ':' ).space( true, true, true )
                )
                .format( '\n', '\0', '\n' ).space( false, false, false )
             << '\n' << '\n';

   /* [results]:

      char[ 3 ][ 3 ][ 3 ] =
      < [ A, 0, 0 ] : [ 0, A, 0 ] : [ 0, 0, A ] >
      < [ B, 0, 0 ] : [ 0, B, 0 ] : [ 0, 0, B ] >
      < [ C, 0, 0 ] : [ 0, C, 0 ] : [ 0, 0, C ] >
   */

   // more complex inlined example
   // [note]: using the generator functions is easier for the more complex constructs
   // as they will deduce the object types for you!

   std::cout << "char[ 3 ][ 3 ][ 3 ] = "
             << boost::io::formatlistfn
                (
                   a3D, a3D + 3, // outermost index
                   boost::io::arrayfmtout
                   (
                      boost::io::arrayfmtex< char * >( 3 ) // innermost index
                      .format( "(", ")" ), // [note]: cannot use characters here; must be char *.
                      3 // middle index
                   )
                   .format( "{\n   ", "\n}", "\n   " ).space( false, false, false )
                )
                .format( "\n", "\0" ).space( false )
             << '\n' << '\n';

   /* [results]:

      char[ 3 ][ 3 ][ 3 ] =
      {
         ( A, 0, 0 )
         ( 0, A, 0 )
         ( 0, 0, A )
      }, {
         ( B, 0, 0 )
         ( 0, B, 0 )
         ( 0, 0, B )
      }, {
         ( C, 0, 0 )
         ( 0, C, 0 )
         ( 0, 0, C )
      }
   */
   
   return( 0 );
}
